@@ -1,6 +1,6 @@
 /*
 32.xlatetof.c - implementation of the elf32_xlateto[fm](3) functions.
-Copyright (C) 1995, 1996 Michael Riepe <michael@stud.uni-hannover.de>
+Copyright (C) 1995 - 1998 Michael Riepe <michael@stud.uni-hannover.de>
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
@@ -21,6 +21,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 #include <ext_types.h>
 #include <byteswap.h>
 
+#ifndef lint
+static const char rcsid[] = "@(#) $Id: 32.xlatetof.c,v 1.6 1998/08/25 15:22:23 michael Exp $";
+#endif /* lint */
+
 /*
  * Ugly, ugly
  */
@@ -28,13 +32,11 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 #if defined/**/x
 # define Cat2(a,b)a##b
 # define Cat3(a,b,c)a##b##c
-# define Exn(m1,m2,args)m1##m2##args
 # define Ex1(m1,m2,a,b)m1##m2(a##b)
 # define Ex2(m1,m2,a,b,c)m1##m2(a,b##c)
 #else
 # define Cat2(a,b)a/**/b
 # define Cat3(a,b,c)a/**/b/**/c
-# define Exn(m1,m2,args)m1/**/m2/**/args
 # define Ex1(m1,m2,a,b)m1/**/m2(a/**/b)
 # define Ex2(m1,m2,a,b,c)m1/**/m2(a,b/**/c)
 #endif
@@ -86,14 +88,14 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 /*
  * scalar copying
  */
-#define copy_scalar_tom(type)	*to = Exn(__load_,type,(*from));
-#define copy_scalar_tof(type)	Exn(__store_,type,(*to, *from));
+#define copy_scalar_tom(type)	*to = Cat2(__load_,type)(*from);
+#define copy_scalar_tof(type)	Cat2(__store_,type)(*to, *from);
 
 /*
  * structure member copying
  */
-#define copy_tom(mb,type)	to->mb = Exn(__load_,type,(from->mb));
-#define copy_tof(mb,type)	Exn(__store_,type,(to->mb, from->mb));
+#define copy_tom(mb,type)	to->mb = Cat2(__load_,type)(from->mb);
+#define copy_tof(mb,type)	Cat2(__store_,type)(to->mb, from->mb);
 
 /*
  * structure member copying (direction independent)
@@ -200,7 +202,7 @@ byte_copy(unsigned char *dst, const unsigned char *src, size_t n) {
 	/* continue with lower half */
 	n /= 2;
     }
-#endif
+#endif /* HAVE_BROKEN_MEMMOVE */
     memmove(dst, src, n);
 }
 
@@ -215,11 +217,11 @@ array_copy(unsigned char *dst, size_t dlen, const unsigned char *src, size_t sle
 /*
  * instantiate copy functions
  */
-copy_type(addr_32,,Elf32_Addr,copy_addr_11)
-copy_type(half_32,,Elf32_Half,copy_half_11)
-copy_type(off_32,,Elf32_Off,copy_off_11)
-copy_type(sword_32,,Elf32_Sword,copy_sword_11)
-copy_type(word_32,,Elf32_Word,copy_word_11)
+copy_type(addr_32,_,Elf32_Addr,copy_addr_11)
+copy_type(half_32,_,Elf32_Half,copy_half_11)
+copy_type(off_32,_,Elf32_Off,copy_off_11)
+copy_type(sword_32,_,Elf32_Sword,copy_sword_11)
+copy_type(word_32,_,Elf32_Word,copy_word_11)
 copy_type(dyn_32,11,Elf32_Dyn,copy_dyn_11)
 copy_type(ehdr_32,11,Elf32_Ehdr,copy_ehdr_11)
 copy_type(phdr_32,11,Elf32_Phdr,copy_phdr_11)
@@ -238,33 +240,33 @@ static const xltab
 xlate32_11[/*encoding*/] = {
     {
 	{ byte_copy,	    byte_copy	    },
-	{ addr_32L_tom,	    addr_32L_tof    },
+	{ addr_32L__tom,    addr_32L__tof   },
 	{ dyn_32L11_tom,    dyn_32L11_tof   },
 	{ ehdr_32L11_tom,   ehdr_32L11_tof  },
-	{ half_32L_tom,	    half_32L_tof    },
-	{ off_32L_tom,	    off_32L_tof	    },
+	{ half_32L__tom,    half_32L__tof   },
+	{ off_32L__tom,	    off_32L__tof    },
 	{ phdr_32L11_tom,   phdr_32L11_tof  },
 	{ rela_32L11_tom,   rela_32L11_tof  },
 	{ rel_32L11_tom,    rel_32L11_tof   },
 	{ shdr_32L11_tom,   shdr_32L11_tof  },
-	{ sword_32L_tom,    sword_32L_tof   },
+	{ sword_32L__tom,   sword_32L__tof  },
 	{ sym_32L11_tom,    sym_32L11_tof   },
-	{ word_32L_tom,	    word_32L_tof    },
+	{ word_32L__tom,    word_32L__tof   },
     },
     {
 	{ byte_copy,	    byte_copy	    },
-	{ addr_32M_tom,	    addr_32M_tof    },
+	{ addr_32M__tom,    addr_32M__tof   },
 	{ dyn_32M11_tom,    dyn_32M11_tof   },
 	{ ehdr_32M11_tom,   ehdr_32M11_tof  },
-	{ half_32M_tom,	    half_32M_tof    },
-	{ off_32M_tom,	    off_32M_tof	    },
+	{ half_32M__tom,    half_32M__tof   },
+	{ off_32M__tom,	    off_32M__tof    },
 	{ phdr_32M11_tom,   phdr_32M11_tof  },
 	{ rela_32M11_tom,   rela_32M11_tof  },
 	{ rel_32M11_tom,    rel_32M11_tof   },
 	{ shdr_32M11_tom,   shdr_32M11_tof  },
-	{ sword_32M_tom,    sword_32M_tof   },
+	{ sword_32M__tom,   sword_32M__tof  },
 	{ sym_32M11_tom,    sym_32M11_tof   },
-	{ word_32M_tom,	    word_32M_tof    },
+	{ word_32M__tom,    word_32M__tof   },
     },
 };
 
@@ -315,8 +317,8 @@ elf32_xlate(Elf_Data *dst, const Elf_Data *src, unsigned encode, int tof) {
 	seterr(ERROR_UNKNOWN_TYPE);
 	return NULL;
     }
-    ssize = _fmsize32(sv, type, 1 - tof);
-    dsize = _fmsize32(dv, type, tof);
+    ssize = _fmsize(ELFCLASS32, sv, type, 1 - tof);
+    dsize = _fmsize(ELFCLASS32, dv, type, tof);
     op = translator(sv, dv, encode, type, tof);
     if (!ssize || !dsize || !op) {
 	seterr(ERROR_UNKNOWN_TYPE);
