@@ -1,6 +1,6 @@
 /*
 newdata.c - implementation of the elf_newdata(3) function.
-Copyright (C) 1995, 1996 Michael Riepe <michael@stud.uni-hannover.de>
+Copyright (C) 1995 - 1998 Michael Riepe <michael@stud.uni-hannover.de>
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
@@ -19,6 +19,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 #include <private.h>
 
+#ifndef lint
+static const char rcsid[] = "@(#) $Id: newdata.c,v 1.4 1998/08/06 16:06:30 michael Exp $";
+#endif /* lint */
+
 Elf_Data*
 elf_newdata(Elf_Scn *scn) {
     Scn_Data *sd;
@@ -35,7 +39,6 @@ elf_newdata(Elf_Scn *scn) {
     }
     else {
 	*sd = _elf_data_init;
-	sd->sd_scn = scn;
 	sd->sd_data_flags = ELF_F_DIRTY;
 	sd->sd_freeme = 1;
 	sd->sd_data.d_version = _elf_version;
@@ -46,7 +49,7 @@ elf_newdata(Elf_Scn *scn) {
 	    scn->s_data_1 = sd;
 	}
 	scn->s_data_n = sd;
-	return (Elf_Data*)sd;
+	return &sd->sd_data;
     }
     return NULL;
 }
