@@ -1,6 +1,6 @@
 /*
 update.c - implementation of the elf_update(3) function.
-Copyright (C) 1995, 1996 Michael Riepe <michael@stud.uni-hannover.de>
+Copyright (C) 1995 - 1998 Michael Riepe <michael@stud.uni-hannover.de>
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
@@ -19,9 +19,13 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 #include <private.h>
 
+#ifndef lint
+static const char rcsid[] = "@(#) $Id: update.c,v 1.6 1998/06/12 19:42:39 michael Exp $";
+#endif /* lint */
+
 #if HAVE_MMAP
 #include <sys/mman.h>
-#endif
+#endif /* HAVE_MMAP */
 
 static const unsigned short __encoding = ELFDATA2LSB + (ELFDATA2MSB << 8);
 #define native_encoding (*(unsigned char*)&__encoding)
@@ -34,6 +38,59 @@ static const unsigned short __encoding = ELFDATA2LSB + (ELFDATA2MSB << 8);
 
 #define max(a,b)		((a)>(b)?(a):(b))
 
+static off_t
+scn_data_layout(Elf_Scn *scn, unsigned v, unsigned type, size_t *algn, unsigned *flag) {
+    Elf *elf = scn->s_elf;
+    int layout = (elf->e_elf_flags & ELF_F_LAYOUT) == 0;
+    size_t scn_align = 1;
+    size_t len = 0;
+    Scn_Data *sd;
+    size_t fsize;
+    size_t msize;
+
+    for (sd = scn->s_data_1; sd; sd = sd->sd_link) {
+	if (!valid_version(sd->sd_data.d_version)) {
+	    return (off_t)-1;
+	}
+
+	if (type != SHT_NOBITS && valid_type(sd->sd_data.d_type)) {
+	    msize = _msize(elf->e_class, sd->sd_data.d_version, sd->sd_data.d_type);
+	    fsize = _fsize(elf->e_class, v, sd->sd_data.d_type);
+	    elf_assert(fsize && msize);
+	    fsize *= sd->sd_data.d_size / msize;
+	}
+	else {
+	    fsize = sd->sd_data.d_size;
+	}
+
+	if (layout) {
+	    align(len, sd->sd_data.d_align);
+	    scn_align = max(scn_align, sd->sd_data.d_align);
+	    rewrite(sd->sd_data.d_off, (off_t)len, sd->sd_data_flags);
+	    len += fsize;
+	}
+	else {
+	    len = max(len, sd->sd_data.d_off + fsize);
+	}
+
+	*flag |= sd->sd_data_flags;
+    }
+    *algn = scn_align;
+    return (off_t)len;
+}
+
+static size_t
+scn_entsize(const Elf *elf, unsigned version, unsigned stype) {
+    if (valid_scntype(stype)) {
+	Elf_Type type = _elf_scn_types[stype];
+
+	if (valid_type(type)) {
+	    return _fsize(elf->e_class, version, type);
+	}
+    }
+    return 0;
+}
+
 static off_t
 _elf32_layout(Elf *elf, unsigned *flag) {
     int layout = (elf->e_elf_flags & ELF_F_LAYOUT) == 0;
@@ -61,16 +118,16 @@ _elf32_layout(Elf *elf, unsigned *flag) {
 	seterr(ERROR_UNKNOWN_ENCODING);
 	return -1;
     }
-    entsize = _fsize32(version, ELF_T_EHDR); elf_assert(entsize);
+    entsize = _fsize(ELFCLASS32, version, ELF_T_EHDR);
+    elf_assert(entsize);
     rewrite(ehdr->e_ehsize, entsize, elf->e_ehdr_flags);
     off = entsize;
 
-    rewrite(ehdr->e_phnum, elf->e_phnum, elf->e_ehdr_flags);
     if (elf->e_phnum) {
-	entsize = _fsize32(version, ELF_T_PHDR); elf_assert(entsize);
-	rewrite(ehdr->e_phentsize, entsize, elf->e_ehdr_flags);
+	entsize = _fsize(ELFCLASS32, version, ELF_T_PHDR);
+	elf_assert(entsize);
 	if (layout) {
-	    align(off, 4);
+	    align(off, _ELF32_ALIGN_PHDR);
 	    rewrite(ehdr->e_phoff, off, elf->e_ehdr_flags);
 	    off += elf->e_phnum * entsize;
 	}
@@ -79,17 +136,18 @@ _elf32_layout(Elf *elf, unsigned *flag) {
 	}
     }
     else {
-	rewrite(ehdr->e_phentsize, 0, elf->e_ehdr_flags);
+	entsize = 0;
 	if (layout) {
 	    rewrite(ehdr->e_phoff, 0, elf->e_ehdr_flags);
 	}
     }
+    rewrite(ehdr->e_phnum, elf->e_phnum, elf->e_ehdr_flags);
+    rewrite(ehdr->e_phentsize, entsize, elf->e_ehdr_flags);
 
     for (scn = elf->e_scn_1, shnum = 0; scn; scn = scn->s_link, ++shnum) {
 	Elf32_Shdr *shdr = &scn->s_shdr32;
 	size_t scn_align = 1;
-	size_t len = 0;
-	Scn_Data *sd;
+	off_t len;
 
 	elf_assert(scn->s_index == shnum);
 
@@ -104,65 +162,31 @@ _elf32_layout(Elf *elf, unsigned *flag) {
 	    }
 	    continue;
 	}
-#if 1
 	if (shdr->sh_type == SHT_NULL) {
 	    continue;
 	}
-#endif
-	for (sd = scn->s_data_1; sd; sd = sd->sd_link) {
-	    size_t fsize, msize;
 
-	    if (shdr->sh_type == SHT_NOBITS) {
-		fsize = sd->sd_data.d_size;
-	    }
-	    else if (!valid_type(sd->sd_data.d_type)) {
-		/* can't translate */
-		fsize = sd->sd_data.d_size;
-	    }
-	    else {
-		msize = _msize32(sd->sd_data.d_version, sd->sd_data.d_type);
-		elf_assert(msize);
-		fsize = _fsize32(version, sd->sd_data.d_type);
-		elf_assert(fsize);
-		fsize = (sd->sd_data.d_size / msize) * fsize;
-	    }
-
-	    if (layout) {
-		align(len, sd->sd_data.d_align);
-		scn_align = max(scn_align, sd->sd_data.d_align);
-		rewrite(sd->sd_data.d_off, (off_t)len, sd->sd_data_flags);
-		len += fsize;
-	    }
-	    else {
-		len = max(len, sd->sd_data.d_off + fsize);
-	    }
-
-	    *flag |= sd->sd_data_flags;
+	len = scn_data_layout(scn, version, shdr->sh_type, &scn_align, flag);
+	if (len == -1) {
+	    return -1;
 	}
 
-	if (valid_scntype(shdr->sh_type)) {
-	    Elf_Type type = _elf_scn_types[shdr->sh_type];
-	    size_t fsize;
-
-	    elf_assert(valid_type(type));
-	    if (type != ELF_T_BYTE) {
-		fsize = _fsize32(version, type);
-		elf_assert(fsize);
-		rewrite(shdr->sh_entsize, fsize, scn->s_shdr_flags);
-	    }
+	entsize = scn_entsize(elf, version, shdr->sh_type);
+	if (entsize > 1) {
+	    rewrite(shdr->sh_entsize, entsize, scn->s_shdr_flags);
 	}
 
 	if (layout) {
 	    align(off, scn_align);
 	    rewrite(shdr->sh_offset, off, scn->s_shdr_flags);
-	    rewrite(shdr->sh_size, len, scn->s_shdr_flags);
+	    rewrite(shdr->sh_size, (size_t)len, scn->s_shdr_flags);
 	    rewrite(shdr->sh_addralign, scn_align, scn->s_shdr_flags);
 
 	    if (shdr->sh_type != SHT_NOBITS) {
-		off += len;
+		off += (size_t)len;
 	    }
 	}
-	else if (len > shdr->sh_size) {
+	else if ((size_t)len > shdr->sh_size) {
 	    seterr(ERROR_SCN2SMALL);
 	    return -1;
 	}
@@ -174,12 +198,11 @@ _elf32_layout(Elf *elf, unsigned *flag) {
 	}
     }
 
-    rewrite(ehdr->e_shnum, shnum, elf->e_ehdr_flags);
     if (shnum) {
-	entsize = _fsize32(version, ELF_T_SHDR); elf_assert(entsize);
-	rewrite(ehdr->e_shentsize, entsize, elf->e_ehdr_flags);
+	entsize = _fsize(ELFCLASS32, version, ELF_T_SHDR);
+	elf_assert(entsize);
 	if (layout) {
-	    align(off, 4);
+	    align(off, _ELF32_ALIGN_SHDR);
 	    rewrite(ehdr->e_shoff, off, elf->e_ehdr_flags);
 	    off += shnum * entsize;
 	}
@@ -188,11 +211,13 @@ _elf32_layout(Elf *elf, unsigned *flag) {
 	}
     }
     else {
-	rewrite(ehdr->e_shentsize, 0, elf->e_ehdr_flags);
+	entsize = 0;
 	if (layout) {
 	    rewrite(ehdr->e_shoff, 0, elf->e_ehdr_flags);
 	}
     }
+    rewrite(ehdr->e_shnum, shnum, elf->e_ehdr_flags);
+    rewrite(ehdr->e_shentsize, entsize, elf->e_ehdr_flags);
 
     rewrite(ehdr->e_ident[EI_MAG0], ELFMAG0, elf->e_ehdr_flags);
     rewrite(ehdr->e_ident[EI_MAG1], ELFMAG1, elf->e_ehdr_flags);
@@ -208,13 +233,158 @@ _elf32_layout(Elf *elf, unsigned *flag) {
     return off;
 }
 
+#if __LIBELF64
+
+static off_t
+_elf64_layout(Elf *elf, unsigned *flag) {
+    int layout = (elf->e_elf_flags & ELF_F_LAYOUT) == 0;
+    Elf64_Ehdr *ehdr = (Elf64_Ehdr*)elf->e_ehdr;
+    size_t off = 0;
+    unsigned version;
+    unsigned encoding;
+    size_t entsize;
+    unsigned shnum;
+    Elf_Scn *scn;
+
+    *flag = elf->e_elf_flags | elf->e_phdr_flags;
+
+    if ((version = ehdr->e_version) == EV_NONE) {
+	version = EV_CURRENT;
+    }
+    if (!valid_version(version)) {
+	seterr(ERROR_UNKNOWN_VERSION);
+	return -1;
+    }
+    if ((encoding = ehdr->e_ident[EI_DATA]) == ELFDATANONE) {
+	encoding = native_encoding;
+    }
+    if (!valid_encoding(encoding)) {
+	seterr(ERROR_UNKNOWN_ENCODING);
+	return -1;
+    }
+    entsize = _fsize(ELFCLASS64, version, ELF_T_EHDR);
+    elf_assert(entsize);
+    rewrite(ehdr->e_ehsize, entsize, elf->e_ehdr_flags);
+    off = entsize;
+
+    if (elf->e_phnum) {
+	entsize = _fsize(ELFCLASS64, version, ELF_T_PHDR);
+	elf_assert(entsize);
+	if (layout) {
+	    align(off, _ELF64_ALIGN_PHDR);
+	    rewrite(ehdr->e_phoff, off, elf->e_ehdr_flags);
+	    off += elf->e_phnum * entsize;
+	}
+	else {
+	    off = max(off, ehdr->e_phoff + elf->e_phnum * entsize);
+	}
+    }
+    else {
+	entsize = 0;
+	if (layout) {
+	    rewrite(ehdr->e_phoff, 0, elf->e_ehdr_flags);
+	}
+    }
+    rewrite(ehdr->e_phnum, elf->e_phnum, elf->e_ehdr_flags);
+    rewrite(ehdr->e_phentsize, entsize, elf->e_ehdr_flags);
+
+    for (scn = elf->e_scn_1, shnum = 0; scn; scn = scn->s_link, ++shnum) {
+	Elf64_Shdr *shdr = &scn->s_shdr64;
+	size_t scn_align = 1;
+	off_t len;
+
+	elf_assert(scn->s_index == shnum);
+
+	*flag |= scn->s_scn_flags | scn->s_shdr_flags;
+
+	if (scn->s_index == SHN_UNDEF) {
+	    rewrite(shdr->sh_entsize, 0, scn->s_shdr_flags);
+	    if (layout) {
+		rewrite(shdr->sh_offset, 0, scn->s_shdr_flags);
+		rewrite(shdr->sh_size, 0, scn->s_shdr_flags);
+		rewrite(shdr->sh_addralign, 0, scn->s_shdr_flags);
+	    }
+	    continue;
+	}
+	if (shdr->sh_type == SHT_NULL) {
+	    continue;
+	}
+
+	len = scn_data_layout(scn, version, shdr->sh_type, &scn_align, flag);
+	if (len == -1) {
+	    return -1;
+	}
+
+	entsize = scn_entsize(elf, version, shdr->sh_type);
+	if (entsize > 1) {
+	    rewrite(shdr->sh_entsize, entsize, scn->s_shdr_flags);
+	}
+
+	if (layout) {
+	    align(off, scn_align);
+	    rewrite(shdr->sh_offset, off, scn->s_shdr_flags);
+	    rewrite(shdr->sh_size, (size_t)len, scn->s_shdr_flags);
+	    rewrite(shdr->sh_addralign, scn_align, scn->s_shdr_flags);
+
+	    if (shdr->sh_type != SHT_NOBITS) {
+		off += (size_t)len;
+	    }
+	}
+	else if ((size_t)len > shdr->sh_size) {
+	    seterr(ERROR_SCN2SMALL);
+	    return -1;
+	}
+	else if (shdr->sh_type != SHT_NOBITS) {
+	    off = max(off, shdr->sh_offset + shdr->sh_size);
+	}
+	else {
+	    off = max(off, shdr->sh_offset);
+	}
+    }
+
+    if (shnum) {
+	entsize = _fsize(ELFCLASS64, version, ELF_T_SHDR);
+	elf_assert(entsize);
+	if (layout) {
+	    align(off, _ELF64_ALIGN_SHDR);
+	    rewrite(ehdr->e_shoff, off, elf->e_ehdr_flags);
+	    off += shnum * entsize;
+	}
+	else {
+	    off = max(off, ehdr->e_shoff + shnum * entsize);
+	}
+    }
+    else {
+	entsize = 0;
+	if (layout) {
+	    rewrite(ehdr->e_shoff, 0, elf->e_ehdr_flags);
+	}
+    }
+    rewrite(ehdr->e_shnum, shnum, elf->e_ehdr_flags);
+    rewrite(ehdr->e_shentsize, entsize, elf->e_ehdr_flags);
+
+    rewrite(ehdr->e_ident[EI_MAG0], ELFMAG0, elf->e_ehdr_flags);
+    rewrite(ehdr->e_ident[EI_MAG1], ELFMAG1, elf->e_ehdr_flags);
+    rewrite(ehdr->e_ident[EI_MAG2], ELFMAG2, elf->e_ehdr_flags);
+    rewrite(ehdr->e_ident[EI_MAG3], ELFMAG3, elf->e_ehdr_flags);
+    rewrite(ehdr->e_ident[EI_CLASS], ELFCLASS64, elf->e_ehdr_flags);
+    rewrite(ehdr->e_ident[EI_DATA], encoding, elf->e_ehdr_flags);
+    rewrite(ehdr->e_ident[EI_VERSION], version, elf->e_ehdr_flags);
+    rewrite(ehdr->e_version, version, elf->e_ehdr_flags);
+
+    *flag |= elf->e_ehdr_flags;
+
+    return off;
+}
+
+#endif /* __LIBELF64 */
+
 #define ptrinside(p,a,l)	((p)>=(a)&&(p)<(a)+(l))
 #define newptr(p,o,n)		((p)=((p)-(o))+(n))
 
 static int
-_elf32_update_pointers(Elf *elf, char *outbuf, size_t len) {
+_elf_update_pointers(Elf *elf, char *outbuf, size_t len) {
     Elf_Scn *scn;
-    Elf32_Shdr *shdr;
     Scn_Data *sd;
     char *data, *rawdata;
 
@@ -241,6 +411,7 @@ _elf32_update_pointers(Elf *elf, char *outbuf, size_t len) {
 	/* update frozen raw image */
 	memcpy(data, outbuf, len);
 	elf->e_data = elf->e_rawdata = data;
+	/* cooked data is stored outside the raw image */
 	return 0;
     }
     if (elf->e_rawdata) {
@@ -277,12 +448,27 @@ _elf32_update_pointers(Elf *elf, char *outbuf, size_t len) {
 	    }
 	}
 	if ((sd = scn->s_rawdata) && sd->sd_memdata && sd->sd_free_data) {
-	    shdr = &scn->s_shdr32;
-	    if (!(rawdata = (char*)realloc(sd->sd_memdata, shdr->sh_size))) {
+	    size_t off, len;
+
+	    if (elf->e_class == ELFCLASS32) {
+		off = scn->s_shdr32.sh_offset;
+		len = scn->s_shdr32.sh_size;
+	    }
+#if __LIBELF64
+	    else if (elf->e_class == ELFCLASS64) {
+		off = scn->s_shdr64.sh_offset;
+		len = scn->s_shdr64.sh_size;
+	    }
+#endif /* __LIBELF64 */
+	    else {
+		seterr(ERROR_UNIMPLEMENTED);
+		return -1;
+	    }
+	    if (!(rawdata = (char*)realloc(sd->sd_memdata, len))) {
 		seterr(ERROR_IO_2BIG);
 		return -1;
 	    }
-	    memcpy(rawdata, outbuf + shdr->sh_offset, shdr->sh_size);
+	    memcpy(rawdata, outbuf + off, len);
 	    if (sd->sd_data.d_buf == sd->sd_memdata) {
 		sd->sd_data.d_buf = rawdata;
 	    }
@@ -293,6 +479,9 @@ _elf32_update_pointers(Elf *elf, char *outbuf, size_t len) {
     return 0;
 }
 
+#undef ptrinside
+#undef newptr
+
 static off_t
 _elf32_write(Elf *elf, char *outbuf, size_t len) {
     Elf32_Ehdr *ehdr;
@@ -305,16 +494,14 @@ _elf32_write(Elf *elf, char *outbuf, size_t len) {
     size_t fsize;
     size_t msize;
 
-    if (!len) {
-	return len;
-    }
-
-    ehdr = (Elf32_Ehdr*)elf->e_ehdr; elf_assert(ehdr);
+    elf_assert(len);
+    elf_assert(elf->e_ehdr);
+    ehdr = (Elf32_Ehdr*)elf->e_ehdr;
     encode = ehdr->e_ident[EI_DATA];
 
     src.d_buf = ehdr;
     src.d_type = ELF_T_EHDR;
-    src.d_size = _msize32(_elf_version, ELF_T_EHDR);
+    src.d_size = _msize(ELFCLASS32, _elf_version, ELF_T_EHDR);
     src.d_version = _elf_version;
     dst.d_buf = outbuf;
     dst.d_size = ehdr->e_ehsize;
@@ -326,7 +513,7 @@ _elf32_write(Elf *elf, char *outbuf, size_t len) {
     if (ehdr->e_phnum) {
 	src.d_buf = elf->e_phdr;
 	src.d_type = ELF_T_PHDR;
-	src.d_size = ehdr->e_phnum * _msize32(_elf_version, ELF_T_PHDR);
+	src.d_size = ehdr->e_phnum * _msize(ELFCLASS32, _elf_version, ELF_T_PHDR);
 	src.d_version = _elf_version;
 	dst.d_buf = outbuf + ehdr->e_phoff;
 	dst.d_size = ehdr->e_phnum * ehdr->e_phentsize;
@@ -337,10 +524,9 @@ _elf32_write(Elf *elf, char *outbuf, size_t len) {
     }
 
     for (scn = elf->e_scn_1; scn; scn = scn->s_link) {
-	shdr = &scn->s_shdr32;
-	src.d_buf = shdr;
+	src.d_buf = &scn->s_uhdr;
 	src.d_type = ELF_T_SHDR;
-	src.d_size = sizeof(*shdr);
+	src.d_size = _msize(ELFCLASS32, EV_CURRENT, ELF_T_SHDR);
 	src.d_version = EV_CURRENT;
 	dst.d_buf = outbuf + ehdr->e_shoff + scn->s_index * ehdr->e_shentsize;
 	dst.d_size = ehdr->e_shentsize;
@@ -352,6 +538,7 @@ _elf32_write(Elf *elf, char *outbuf, size_t len) {
 	if (scn->s_index == SHN_UNDEF) {
 	    continue;
 	}
+	shdr = &scn->s_shdr32;
 	if (shdr->sh_type == SHT_NULL || shdr->sh_type == SHT_NOBITS) {
 	    continue;
 	}
@@ -371,9 +558,9 @@ _elf32_write(Elf *elf, char *outbuf, size_t len) {
 	    dst.d_size = src.d_size;
 	    dst.d_version = ehdr->e_version;
 	    if (valid_type(src.d_type)) {
-		msize = _msize32(src.d_version, src.d_type);
+		msize = _msize(ELFCLASS32, src.d_version, src.d_type);
 		elf_assert(msize);
-		fsize = _fsize32(dst.d_version, src.d_type);
+		fsize = _fsize(ELFCLASS32, dst.d_version, src.d_type);
 		elf_assert(fsize);
 		if (msize != fsize) {
 		    dst.d_size = (src.d_size / msize) * fsize;
@@ -389,7 +576,7 @@ _elf32_write(Elf *elf, char *outbuf, size_t len) {
     }
 
     /* cleanup */
-    if (elf->e_readable && _elf32_update_pointers(elf, outbuf, len)) {
+    if (elf->e_readable && _elf_update_pointers(elf, outbuf, len)) {
 	return -1;
     }
     /* NOTE: ehdr is no longer valid! */
@@ -416,12 +603,193 @@ _elf32_write(Elf *elf, char *outbuf, size_t len) {
     return len;
 }
 
+#if __LIBELF64
+
+static off_t
+_elf64_write(Elf *elf, char *outbuf, size_t len) {
+    Elf64_Ehdr *ehdr;
+    Elf64_Shdr *shdr;
+    Elf_Scn *scn;
+    Scn_Data *sd;
+    Elf_Data src;
+    Elf_Data dst;
+    unsigned encode;
+    size_t fsize;
+    size_t msize;
+
+    elf_assert(len);
+    elf_assert(elf->e_ehdr);
+    ehdr = (Elf64_Ehdr*)elf->e_ehdr;
+    encode = ehdr->e_ident[EI_DATA];
+
+    src.d_buf = ehdr;
+    src.d_type = ELF_T_EHDR;
+    src.d_size = _msize(ELFCLASS64, _elf_version, ELF_T_EHDR);
+    src.d_version = _elf_version;
+    dst.d_buf = outbuf;
+    dst.d_size = ehdr->e_ehsize;
+    dst.d_version = ehdr->e_version;
+    if (!elf64_xlatetof(&dst, &src, encode)) {
+	return -1;
+    }
+
+    if (ehdr->e_phnum) {
+	src.d_buf = elf->e_phdr;
+	src.d_type = ELF_T_PHDR;
+	src.d_size = ehdr->e_phnum * _msize(ELFCLASS64, _elf_version, ELF_T_PHDR);
+	src.d_version = _elf_version;
+	dst.d_buf = outbuf + ehdr->e_phoff;
+	dst.d_size = ehdr->e_phnum * ehdr->e_phentsize;
+	dst.d_version = ehdr->e_version;
+	if (!elf64_xlatetof(&dst, &src, encode)) {
+	    return -1;
+	}
+    }
+
+    for (scn = elf->e_scn_1; scn; scn = scn->s_link) {
+	src.d_buf = &scn->s_uhdr;
+	src.d_type = ELF_T_SHDR;
+	src.d_size = _msize(ELFCLASS64, EV_CURRENT, ELF_T_SHDR);
+	src.d_version = EV_CURRENT;
+	dst.d_buf = outbuf + ehdr->e_shoff + scn->s_index * ehdr->e_shentsize;
+	dst.d_size = ehdr->e_shentsize;
+	dst.d_version = ehdr->e_version;
+	if (!elf64_xlatetof(&dst, &src, encode)) {
+	    return -1;
+	}
+
+	if (scn->s_index == SHN_UNDEF) {
+	    continue;
+	}
+	shdr = &scn->s_shdr64;
+	if (shdr->sh_type == SHT_NULL || shdr->sh_type == SHT_NOBITS) {
+	    continue;
+	}
+	if (scn->s_data_1 && !elf_getdata(scn, NULL)) {
+	    return -1;
+	}
+	for (sd = scn->s_data_1; sd; sd = sd->sd_link) {
+	    src = sd->sd_data;
+	    if (!src.d_size) {
+		continue;
+	    }
+	    if (!src.d_buf) {
+		seterr(ERROR_NULLBUF);
+		return -1;
+	    }
+	    dst.d_buf = outbuf + shdr->sh_offset + src.d_off;
+	    dst.d_size = src.d_size;
+	    dst.d_version = ehdr->e_version;
+	    if (valid_type(src.d_type)) {
+		msize = _msize(ELFCLASS64, src.d_version, src.d_type);
+		elf_assert(msize);
+		fsize = _fsize(ELFCLASS64, dst.d_version, src.d_type);
+		elf_assert(fsize);
+		if (msize != fsize) {
+		    dst.d_size = (src.d_size / msize) * fsize;
+		}
+	    }
+	    else {
+		src.d_type = ELF_T_BYTE;
+	    }
+	    if (!elf64_xlatetof(&dst, &src, encode)) {
+		return -1;
+	    }
+	}
+    }
+
+    /* cleanup */
+    if (elf->e_readable && _elf_update_pointers(elf, outbuf, len)) {
+	return -1;
+    }
+    /* NOTE: ehdr is no longer valid! */
+    ehdr = (Elf64_Ehdr*)elf->e_ehdr; elf_assert(ehdr);
+    elf->e_encoding = ehdr->e_ident[EI_DATA];
+    elf->e_version = ehdr->e_ident[EI_VERSION];
+    elf->e_elf_flags &= ~ELF_F_DIRTY;
+    elf->e_ehdr_flags &= ~ELF_F_DIRTY;
+    elf->e_phdr_flags &= ~ELF_F_DIRTY;
+    for (scn = elf->e_scn_1; scn; scn = scn->s_link) {
+	scn->s_scn_flags &= ~ELF_F_DIRTY;
+	scn->s_shdr_flags &= ~ELF_F_DIRTY;
+	for (sd = scn->s_data_1; sd; sd = sd->sd_link) {
+	    sd->sd_data_flags &= ~ELF_F_DIRTY;
+	}
+	if (elf->e_readable) {
+	    shdr = &scn->s_shdr64;
+	    scn->s_type = shdr->sh_type;
+	    scn->s_size = shdr->sh_size;
+	    scn->s_offset = shdr->sh_offset;
+	}
+    }
+    elf->e_size = len;
+    return len;
+}
+
+#endif /* __LIBELF64 */
+
+static off_t
+_elf_output(Elf *elf, size_t len, off_t (*_elf_write)(Elf*, char*, size_t)) {
+    char *buf;
+    off_t err;
+
+    elf_assert(len);
+#if HAVE_FTRUNCATE
+    ftruncate(elf->e_fd, 0);
+#endif /* HAVE_FTRUNCATE */
+#if HAVE_MMAP
+    /*
+     * Make sure the file is (at least) len bytes long
+     */
+#if HAVE_FTRUNCATE
+    if (ftruncate(elf->e_fd, len)) {
+#else /* HAVE_FTRUNCATE */
+    {
+#endif /* HAVE_FTRUNCATE */
+	if (lseek(elf->e_fd, (off_t)len - 1, 0) != (off_t)len - 1) {
+	    seterr(ERROR_IO_SEEK);
+	    return -1;
+	}
+	if (write(elf->e_fd, "", 1) != 1) {
+	    seterr(ERROR_IO_WRITE);
+	    return -1;
+	}
+    }
+    buf = (void*)mmap(NULL, len, PROT_READ | PROT_WRITE, MAP_SHARED,
+		      elf->e_fd, 0);
+    if (buf != (char*)-1) {
+	if ((char)_elf_fill) {
+	    memset(buf, _elf_fill, len);
+	}
+	err = _elf_write(elf, buf, len);
+	munmap(buf, len);
+	return err;
+    }
+#endif /* HAVE_MMAP */
+    if (!(buf = (char*)malloc(len))) {
+	seterr(ERROR_MEM_OUTBUF);
+	return -1;
+    }
+    memset(buf, _elf_fill, len);
+    err = _elf_write(elf, buf, len);
+    if (err != -1 && (size_t)err == len) {
+	if (lseek(elf->e_fd, (off_t)0, 0)) {
+	    seterr(ERROR_IO_SEEK);
+	    err = -1;
+	}
+	else if (write(elf->e_fd, buf, len) != len) {
+	    seterr(ERROR_IO_WRITE);
+	    err = -1;
+	}
+    }
+    free(buf);
+    return err;
+}
+
 off_t
 elf_update(Elf *elf, Elf_Cmd cmd) {
     unsigned flag;
     off_t len;
-    char *buf;
-    int err;
 
     if (!elf) {
 	return -1;
@@ -450,64 +818,20 @@ elf_update(Elf *elf, Elf_Cmd cmd) {
     }
     else if (elf->e_class == ELFCLASS32) {
 	len = _elf32_layout(elf, &flag);
-	if (len == -1 || cmd != ELF_C_WRITE || !(flag & ELF_F_DIRTY)) {
-	    return len;
+	if (len != -1 && cmd == ELF_C_WRITE && (flag & ELF_F_DIRTY)) {
+	    len = _elf_output(elf, (size_t)len, _elf32_write);
 	}
-	if (!len) {
-	    /* can this happen at all??? */
-	    return len;
-	}
-#if HAVE_FTRUNCATE
-	ftruncate(elf->e_fd, 0);
-#endif
-#if HAVE_MMAP
-	/*
-	 * Make sure the file is (at least) len bytes long
-	 */
-#if HAVE_FTRUNCATE
-	if (ftruncate(elf->e_fd, len)) {
-#else
-	{
-#endif
-	    if (lseek(elf->e_fd, (long)len - 1, 0) != (long)len - 1) {
-		seterr(ERROR_IO_SEEK);
-		return -1;
-	    }
-	    if (write(elf->e_fd, "", 1) != 1) {
-		seterr(ERROR_IO_WRITE);
-		return -1;
-	    }
-	}
-	buf = (void*)mmap(NULL, len, PROT_READ | PROT_WRITE, MAP_SHARED,
-			  elf->e_fd, 0);
-	if (buf != (char*)-1) {
-	    if ((char)_elf_fill) {
-		memset(buf, _elf_fill, len);
-	    }
-	    err = _elf32_write(elf, buf, len);
-	    munmap(buf, len);
-	    return err;
-	}
-#endif
-	if (!(buf = (char*)malloc(len))) {
-	    seterr(ERROR_MEM_OUTBUF);
-	    return -1;
-	}
-	memset(buf, _elf_fill, len);
-	err = _elf32_write(elf, buf, len);
-	if (err == len) {
-	    if (lseek(elf->e_fd, 0L, 0)) {
-		seterr(ERROR_IO_SEEK);
-		err = -1;
-	    }
-	    else if (write(elf->e_fd, buf, len) != len) {
-		seterr(ERROR_IO_WRITE);
-		err = -1;
-	    }
-	}
-	free(buf);
-	return err;
+	return len;
     }
+#if __LIBELF64
+    else if (elf->e_class == ELFCLASS64) {
+	len = _elf64_layout(elf, &flag);
+	if (len != -1 && cmd == ELF_C_WRITE && (flag & ELF_F_DIRTY)) {
+	    len = _elf_output(elf, len, _elf64_write);
+	}
+	return len;
+    }
+#endif /* __LIBELF64 */
     else if (valid_class(elf->e_class)) {
 	seterr(ERROR_UNIMPLEMENTED);
     }
